@@ -2,6 +2,7 @@ from abc import abstractmethod, ABC
 from collections import UserDict, UserString
 from collections.abc import Mapping
 from contextlib import contextmanager
+from io import StringIO
 #
 from .answers import AnswersBlock, SubquestionsAnswer
 from .config import ConfigParser
@@ -52,6 +53,14 @@ class _QuestionsContainerBase(Component):
         if register is True:
             qform.blocks[name] = self
 
+    def __str__(self):
+        """Return full name"""
+        return self.name
+
+    def __repr__(self):
+        """Return full name"""
+        return self.name
+
     @property
     def label(self):
         return self._name
@@ -157,10 +166,7 @@ class LiteralBlock(_ConcreteQuestionBase):
 
     @property
     def answer(self):
-        answer = self.get_answer()
-        if answer is NOT_DEFINED:
-            return ""
-        return LiteralBlockString(answer)
+        return LiteralBlockString(self.get_answer())
 
     @answer.setter
     def answer(self, value):
@@ -182,7 +188,7 @@ class LiteralBlock(_ConcreteQuestionBase):
     def get_answer_as_string(self):
         """get string of answer"""
         if self._answer.is_none is True:
-            return None
+            return ''
         return self._answer
 
     def accept(self, visitor):
@@ -237,7 +243,7 @@ class ConcreteQuestion(_ConcreteQuestionBase):
     def get_answer_as_string(self):
         """get answer back, if is optional, return None if NOT_DEFINED"""
         if self.is_set_to_empty is True:
-            return None
+            return ''
         return self._value.answer_as_string()
 
     def accept(self, visitor):
@@ -379,36 +385,6 @@ class SubquestionBlock(_QuestionsContainerBase):
         return self.cases[answer].concrete
 
 
-def generate_string(name, value):
-    if value is None:
-        return f"{name} ="
-    return f"{name} = {value}"
-
-
-def answer_iter(name, dct, default_name):
-    if dct is None:
-        return
-
-    if isinstance(dct, LiteralBlockString):
-        if dct.is_none is True:
-            return
-    else:
-        if len(dct) == 0:
-            return
-
-    if name != default_name:
-        yield f'[{name}]'
-    else:
-        yield ''
-
-    if isinstance(dct, LiteralBlockString):
-        yield dct.data
-    else:
-        for _name, _value in dct.items():
-            yield generate_string(_name, _value)
-        yield ''
-
-
 class QuestionVisitor(ABC):
     """Base class to define visitors for the question form
     the entry point is always the `QuestionForm`"""
@@ -486,6 +462,8 @@ def block_not_set(block_name, keys):
 class ColtErrorAnswerNotDefined(SystemExit):
     """Error if answer is not defined"""
 
+    __slots__ = ()
+
     def __init__(self, msg):
         super().__init__(f"ColtErrorAnswerNotDefined:\n{msg}")
 
@@ -682,6 +660,8 @@ class QuestionGeneratorVisitor(QuestionASTVisitor):
 class ErrorSettingAnswerFromFile(SystemExit):
     """errors setting answers from file"""
 
+    __slots__ = ()
+
     def __init__(self, filename, msg):
         super().__init__(f"ErrorSettingAnswerFromFile: file = '{filename}'\n{msg}")
 
@@ -690,16 +670,64 @@ class ErrorSettingAnswerFromFile(SystemExit):
 class ErrorSettingAnswerFromDict(SystemExit):
     """Error when trying to read answers from a dict"""
 
+    __slots__ = ()
+
     def __init__(self, msg):
         super().__init__(f"ErrorSettingAnswerFromDict:\n{msg}")
 
 
+class WriteAnswerVisitor(QuestionVisitor):
+    """Visitor to write the answers to a file"""
+
+    __slots__ = ('txt')
+
+    def visit_qform(self, qform, **kwargs):
+        self.txt = ''
+        for blockname in qform.get_blocks():
+            # normal blocks
+            qform[blockname].accept(self)
+        return self.txt
+
+    def visit_question_block(self, block):
+        if block.name != '':
+            self.txt += f'\n[{block.name}]\n'
+        # first all normal questions
+        for question in block.concrete.values():
+            if not isinstance(question, LiteralBlock):
+                question.accept(self)
+        # than literal blocks
+        for question in block.concrete.values():
+            if isinstance(question, LiteralBlock):
+                question.accept(self)
+
+    def visit_concrete_question_select(self, question):
+        self.txt += f'{question.short_name} = {question.get_answer_as_string()}\n'
+
+    def visit_concrete_question_hidden(self, question):
+        pass
+
+    def visit_concrete_question_input(self, question):
+        self.txt += f'{question.short_name} = {question.get_answer_as_string()}\n'
+
+    def visit_literal_block(self, block):
+        answer = block.answer
+        if answer.is_none is True:
+            return
+        self.txt += f'[{block.id}]\n{answer}\n'
+
+    def visit_subquestion_block(self, block):
+        """visit subquestion blocks"""
+        raise Exception("should never arrive in subquestion block!")
+
+
 class QuestionForm(Mapping, Component):
     """Main interface to the question forms"""
     #
     __slots__ = ('blocks', 'literals', 'unset', 'form')
     # visitor to generate answers
     answer_visitor = AnswerVisitor()
+    # visitor to write answers to file
+    write_visitor = WriteAnswerVisitor()
     # visitor to generate question forms
     question_generator_visitor = QuestionGeneratorVisitor()
 
@@ -772,21 +800,10 @@ class QuestionForm(Mapping, Component):
 
     def write_config(self, filename):
         """ get a linear config and write it to the file"""
-        config = {}
-        for blockname in self.get_blocks():
-            # normal blocks
-            config[blockname] = {key: question.get_answer_as_string()
-                                 for key, question in self.blocks[blockname].concrete.items()
-                                 if not isinstance(question, LiteralBlock)}
-            # add literal blocks
-            config.update({question.id: question.get_answer_as_string()
-                           for _, question in self.blocks[blockname].concrete.items()
-                           if isinstance(question, LiteralBlock)})
-
-        default_name = ''
+        if isinstance(filename, StringIO):
+            return
         with open(filename, 'w') as fhandle:
-            fhandle.write("\n".join(answer for key, answers in config.items()
-                                    for answer in answer_iter(key, answers, default_name)))
+            fhandle.write(self.write_visitor.visit(self))
 
     def set_answers_from_file(self, filename, raise_error=True):
         errmsg = self._set_answers_from_file(filename)
@@ -806,7 +823,7 @@ class QuestionForm(Mapping, Component):
         if config is not None:
             if isinstance(config, Mapping):
                 self.set_answers_from_dct(config, raise_error=raise_error)
-            elif is_existing_file(config):
+            elif isinstance(config, StringIO) or is_existing_file(config):
                 self.set_answers_from_file(config, raise_error=raise_error)
 
     def set_presets(self, presets):
@@ -884,7 +901,7 @@ class QuestionForm(Mapping, Component):
         block = self.blocks[blockname]
         for key, answer in answers.items():
             if key not in block:
-                print(f"key '{key}' in block '{block}'  not known")
+                print(f"unknown key '{key}' in '[{block}]'")
                 continue
             question = block[key]
             if answer == "":
